@@ -317,6 +317,24 @@ def add_related_assets(public_id, assets_to_relate, resource_type="image", type=
     return call_json_api("post", uri, params, **options)
 
 
+def add_related_assets_by_asset_ids(asset_id, assets_to_relate, **options):
+    """
+    Relates an asset to other assets by asset IDs.
+
+    :param asset_id: The asset ID of the asset to update.
+    :type asset_id: str
+    :param assets_to_relate: The array of up to 10 asset IDs.
+    :type assets_to_relate: list[str]
+    :param options: Additional options.
+    :type options: dict, optional
+    :return: The result of the command.
+    :rtype: dict
+    """
+    uri = ["resources", "related_assets", asset_id]
+    params = {"assets_to_relate": utils.build_array(assets_to_relate)}
+    return call_json_api("post", uri, params, **options)
+
+
 def delete_related_assets(public_id, assets_to_unrelate, resource_type="image", type="upload", **options):
     """
     Unrelates an asset from other assets by public IDs.
@@ -339,6 +357,24 @@ def delete_related_assets(public_id, assets_to_unrelate, resource_type="image",
     return call_json_api("delete", uri, params, **options)
 
 
+def delete_related_assets_by_asset_ids(asset_id, assets_to_unrelate, **options):
+    """
+    Unrelates an asset from other assets by asset IDs.
+
+    :param asset_id: The asset ID of the asset to update.
+    :type asset_id: str
+    :param assets_to_unrelate: The array of up to 10 asset IDs.
+    :type assets_to_unrelate: list[str]
+    :param options: Additional options.
+    :type options: dict, optional
+    :return: The result of the command.
+    :rtype: dict
+    """
+    uri = ["resources", "related_assets", asset_id]
+    params = {"assets_to_unrelate": utils.build_array(assets_to_unrelate)}
+    return call_json_api("delete", uri, params, **options)
+
+
 def tags(**options):
     resource_type = options.pop("resource_type", "image")
     uri = ["tags", resource_type]
