@@ -11,7 +11,7 @@ import cloudinary
 from cloudinary import api
 from cloudinary.exceptions import BadRequest, NotFound
 from test.helper_test import (
-    UNIQUE_TEST_ID, get_uri, get_params, get_method, api_response_mock, ignore_exception
+    UNIQUE_TEST_ID, get_uri, get_params, get_method, api_response_mock, ignore_exception, get_json_body
 )
 
 MOCK_RESPONSE = api_response_mock()
@@ -345,20 +345,6 @@ class MetadataTest(unittest.TestCase):
 
         self.assertEqual(json.loads(kargs["body"].decode('utf-8')), {})
 
-    @unittest.skipUnless(cloudinary.config().api_secret, "requires api_key/api_secret")
-    def test11_delete_metadata_field_does_not_release_external_id(self):
-        """Test deleting a metadata field definition then attempting to create a
-        new one with the same external id which should fail.
-        """
-        api.delete_metadata_field(EXTERNAL_ID_DELETE_2)
-
-        with self.assertRaises(BadRequest):
-            api.add_metadata_field({
-                "external_id": EXTERNAL_ID_DELETE_2,
-                "label": EXTERNAL_ID_DELETE_2,
-                "type": "integer",
-            })
-
     @unittest.skipUnless(cloudinary.config().api_secret, "requires api_key/api_secret")
     def test12_delete_metadata_field_data_source(self):
         """Delete entries in a metadata field datasource"""
@@ -473,6 +459,72 @@ class MetadataTest(unittest.TestCase):
         self.assert_metadata_field_datasource(result)
         self.assertEqual(len(result["values"]), 3)
 
+    @unittest.skipUnless(cloudinary.config().api_secret, "requires api_key/api_secret")
+    def test_order_by_asc_by_default_in_a_metadata_field_data_source(self):
+        # datasource is set with values in the order v2, v3, v4
+        result = api.reorder_metadata_field_datasource(EXTERNAL_ID_SET_3, 'value')
+
+        self.assert_metadata_field_datasource(result)
+
+        self.assertEqual(result['values'][0]['value'], DATASOURCE_MULTIPLE[0]['value'])
+
+    @unittest.skipUnless(cloudinary.config().api_secret, "requires api_key/api_secret")
+    def test_order_by_asc_in_a_metadata_field_data_source(self):
+        # datasource is set with values in the order v2, v3, v4
+        result = api.reorder_metadata_field_datasource(EXTERNAL_ID_SET_3, 'value', 'asc')
+
+        self.assert_metadata_field_datasource(result)
+
+        self.assertEqual(result['values'][0]['value'], DATASOURCE_MULTIPLE[0]['value'])
+
+    @unittest.skipUnless(cloudinary.config().api_secret, "requires api_key/api_secret")
+    def test_order_by_desc_in_a_metadata_field_data_source(self):
+        # datasource is set with values in the order v2, v3, v4
+        result = api.reorder_metadata_field_datasource(EXTERNAL_ID_SET_3, 'value', 'desc')
+
+        self.assert_metadata_field_datasource(result)
+
+        self.assertEqual(result['values'][0]['value'], DATASOURCE_MULTIPLE[-1]['value'])
+
+    @patch("urllib3.request.RequestMethods.request")
+    @unittest.skipUnless(cloudinary.config().api_secret, "requires api_key/api_secret")
+    def test_reorder_metadata_fields_by_label(self, mocker):
+        """Test the reorder of metadata fields for label order by asc"""
+        mocker.return_value = MOCK_RESPONSE
+        api.reorder_metadata_fields('label', 'asc')
+        args, kargs = mocker.call_args
+
+        self.assertTrue(get_uri(args).endswith("/metadata_fields/order"))
+        self.assertEqual(get_method(mocker), "PUT")
+        self.assertEqual(get_json_body(mocker)['order_by'], "label")
+        self.assertEqual(get_json_body(mocker)['direction'], "asc")
+
+    @patch("urllib3.request.RequestMethods.request")
+    @unittest.skipUnless(cloudinary.config().api_secret, "requires api_key/api_secret")
+    def test_reorder_metadata_fields_by_external_id(self, mocker):
+        """Test the reorder of metadata fields for external_id order by desc"""
+        mocker.return_value = MOCK_RESPONSE
+        api.reorder_metadata_fields('external_id', 'desc')
+        args, kargs = mocker.call_args
+
+        self.assertTrue(get_uri(args).endswith("/metadata_fields/order"))
+        self.assertEqual(get_method(mocker), "PUT")
+        self.assertEqual(get_json_body(mocker)['order_by'], "external_id")
+        self.assertEqual(get_json_body(mocker)['direction'], "desc")
+
+    @patch("urllib3.request.RequestMethods.request")
+    @unittest.skipUnless(cloudinary.config().api_secret, "requires api_key/api_secret")
+    def test_reorder_metadata_fields_by_created_at(self, mocker):
+        """Test the reorder of metadata fields for created_at order by asc"""
+        mocker.return_value = MOCK_RESPONSE
+        api.reorder_metadata_fields('created_at', 'asc')
+        args, kargs = mocker.call_args
+
+        self.assertTrue(get_uri(args).endswith("/metadata_fields/order"))
+        self.assertEqual(get_method(mocker), "PUT")
+        self.assertEqual(get_json_body(mocker)['order_by'], "created_at")
+        self.assertEqual(get_json_body(mocker)['direction'], "asc")
+
 
 if __name__ == "__main__":
     unittest.main()
